@@ -33,7 +33,7 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
     _rec_name = 'description'
 
     party = fields.Many2One('party.party', 'Party', required=True, select=1,
-        on_change=['party'], states=_STATES_STOP, depends=['state'])
+            states=_STATES_STOP, depends=['state'])
     address = fields.Many2One('party.address', 'Address',
             domain=[('party', '=', Eval('party'))],
             select=2, depends=['party', 'state'],
@@ -172,8 +172,8 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
         res = {
             'payment_term': False,
         }
-        if values.get('party'):
-            party = party_obj.browse(values['party'])
+        if vals.get('party'):
+            party = party_obj.browse(vals['party'])
             if party.payment_term:
                 res['payment_term'] = party.payment_term.id
         if not res['payment_term']:
@@ -364,18 +364,6 @@ class SaleOpportunityLine(ModelSQL, ModelView):
 SaleOpportunityLine()
 
 
-class One2ManyHistory(fields.One2Many):
-
-    def get(self, ids, model, name, values=None):
-        opportunity2id = {}
-        histories = model.browse(ids)
-        opportunity2id = dict((history.opportunity.id, history.id) for
-                history in histories)
-        res = super(One2ManyHistory, self).get(opportunity2id.keys(), model,
-                name, values=values)
-        return dict((opportunity2id[i], j) for i, j in res.iteritems())
-
-
 class SaleOpportunityHistory(ModelSQL, ModelView):
     'Sale Opportunity History'
     _name = 'sale.opportunity.history'
@@ -398,8 +386,8 @@ class SaleOpportunityHistory(ModelSQL, ModelView):
     }, depends=['state'])
     description = fields.Char('Description')
     comment = fields.Text('Comment')
-    lines = One2ManyHistory('sale.opportunity.line', 'opportunity', 'Lines',
-            datetime_field='date')
+    lines = fields.Function(fields.One2Many('sale.opportunity.line', None, 'Lines',
+            datetime_field='date'), 'get_lines')
     state = fields.Selection(STATES, 'State')
     probability = fields.Integer('Conversion Probability')
     lost_reason = fields.Text('Reason for loss', states={
@@ -443,6 +431,18 @@ class SaleOpportunityHistory(ModelSQL, ModelView):
                 (', '.join(self._table_query_group()))) % \
                 opportunity_obj._table, [])
 
+    def get_lines(self, ids, name):
+        line_obj = self.pool.get('sale.opportunity.line')
+        histories = self.browse(ids)
+        result = {}
+        # We will always have only one id per call due to datetime_field
+        for history in histories:
+            result[history.id] = line_obj.search([
+                ('opportunity', '=', history.opportunity.id),
+            ])
+        return result
+
+
     def read(self, ids, fields_names=None):
         res = super(SaleOpportunityHistory, self).read(ids,
                 fields_names=fields_names)
